/***************************************************************************
                          gmx_rescue.c  -  description
                             -------------------
    begin                : Fri Apr  5 20:17:10 BST 2002
    origin               : from postings on Gromacs Mailing list by
                           Bert de Groot and Peter Tieleman
    modifications        : (C) 2003 by Marc Baaden, 
                               2004 Oliver Beckstein (large file support)
    email                : baaden@smplinux.de

    compile (Linux): gcc -DLINUX -D_LARGEFILE64_SOURCE -o gmx_rescue gmx_rescue.c
    

 ***************************************************************************/
/*
 * $Id: gmx_rescue.c,v 1.6 2004/09/12 10:05:15 marc Exp $
 *
 *                This source code is part of
 *
 *                 G   R   O   M   A   C   S
 *
 *          GROningen MAchine for Chemical Simulations
 *
 *                        VERSION 3.0
 *
 * Copyright (c) 1991-2001
 * BIOSON Research Institute, Dept. of Biophysical Chemistry
 * University of Groningen, The Netherlands
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * If you want to redistribute modifications, please consider that
 * scientific software is very special. Version control is crucial -
 * bugs must be traceable. We will be happy to consider code for
 * inclusion in the official distribution, but derived work must not
 * be called official GROMACS. Details are found in the README & COPYING
 * files - if they are missing, get the official version at www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the papers on the package - you can find them in the top README file.
 *
 * Do check out http://www.gromacs.org , or mail us at gromacs@gromacs.org .
 *
 * And Hey:
 * Gyas ROwers Mature At Cryogenic Speed
 */

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

// Linux magic number
#ifdef LINUX
#define MAGIC_NUMBER -888733696
#endif

// SGI magic number
#ifdef SGI
#define MAGIC_NUMBER 1995
#endif

int main(int argc,  char *argv[]) {

    int f, out;
    int value;
    int res, n, ln, frames,offset;
    char buf[1024*32];

        fprintf(stderr, "%s  $Id: gmx_rescue.c,v 1.6 2004/09/12 10:05:15 marc Exp $\n",argv[0]);
#ifdef _LARGEFILE64_SOURCE
        fprintf(stderr, "(compiled with large file support)\n");
#else
        fprintf(stderr, "gmx_rescue (no large file support compiled !)\n");
#endif

    if (argc!=3 && argc !=4) {
        fprintf(stderr, "usage: %s <filename> scan: finds magic nunmbers\n", argv[0]);
        fprintf(stderr, "usage: %s <filename> <rescue file> <offset>: rescues starting from offset\n", argv[0]);
	return 10;
    }

    // open xtc file readonly
#ifdef _LARGEFILE64_SOURCE
    f = open(argv[1], O_RDONLY|O_LARGEFILE);
#else
    f = open(argv[1], O_RDONLY);
#endif
    if (f == -1) {
        perror("couldn't open file for reading");
	return 20;
    }
    
    // Scan for magic number positions
    if (strcmp(argv[2],"scan")==0) {
      printf("Frame  Delta  Offset\n");
      ln=0; n=0; frames=0; res=1;
      while (res > 0) {
        while ((res=read(f, &value, 4)) == 4 && value != MAGIC_NUMBER) {
          n++;
        }
        printf("%5d  %5d  %d\n",frames,n-ln,n);
        frames++;
        ln=n;
      }
      return 0;
    }
    
    offset = 4 * atoi(argv[3]);

    // reposition file offset by setting it to offset bytes
    lseek(f, offset, SEEK_SET);

    // read up to 4 bytes from file into value until magic number is found
    while (read(f, &value, 4) == 4 && value != MAGIC_NUMBER) {
      ;
    }

    //Linux problem for position reporting: we work on a file descriptor,
    //not a stream. But we only have access to ftell !
    //printf("postion %ld \n", ftell(f)-4);

#ifdef SGI
    printf("postion %ld \n", (long)tell(f)-4);
#endif
    
    //Attention: file L:
#ifdef _LARGEFILE64_SOURCE
    out=open(argv[2], O_WRONLY | O_CREAT | O_EXCL | O_LARGEFILE, 0666);
#else
    out=open(argv[2], O_WRONLY | O_CREAT | O_EXCL, 0666);
#endif
    if (out == -1) {
        perror("couldn't open file for writing");
        printf("Please delete the file first, if it exists already !\n");
	return 30;
    }
    write(out, &value, 4);
    while ((res = read(f, buf, 1024*32)) > 0) {
       write(out, buf, res);
    }
    
    return 0;
}
